#modules/make_questions_options.py
from pathlib import Path
import customtkinter
import tkinter as tk
import json
import os
import sys  
from tkinter import messagebox
from openpyxl import load_workbook
from customtkinter import CTkToplevel, CTkFrame, CTkButton, CTkLabel, CTkScrollableFrame, CTkEntry
from .copymanager import CopyManager



class ManualQuestionPopup문제선택(CTkToplevel): #WINDOWS
    def __init__(self, parent, main_frame_ref=None, excel_file=None, default_font=None, passage_ids=None, 유형=None, 난이도=None):
        super().__init__(parent)

        print("ManualQuestionPopup문제선택 called")
        print(f"passage_ids: {passage_ids}")

        #self.process_excel_callback = process_excel_callback
        self.title("수동으로 출제할 문제 선택")
        self.geometry("580x400")
        self.minsize(580, 400)
        print("ManualQuestionPopup문제선택 called")

        self.main_frame_ref = main_frame_ref  # Store the reference to main frame
        self.excel_file = excel_file
        self.default_font = default_font

        self.current_status = "자동"

        # If passage_ids is a multi-line string, convert it to a list of lines
        if isinstance(passage_ids, str):
            passage_ids = passage_ids.strip().split('\n')
        self.passage_ids = passage_ids
        self.유형 = 유형
        self.난이도 = 난이도    



        # Define column widths as class constants
        self.COL_WIDTHS = {
            "번호": 40,
            "지문번호": 200,
            "문제유형": 80,
            "출제방식": 80,
            "수동출제": 100
        }



        # Container Frame
        main_frame = CTkFrame(self, fg_color="transparent")
        main_frame.pack(fill="both", expand=True, padx=10, pady=10)
        main_frame.grid_rowconfigure(1, weight=1)
        main_frame.grid_columnconfigure(0, weight=1)

        # Header
        # Header
        header_frame = CTkFrame(main_frame, fg_color="transparent")
        header_frame.grid(row=0, column=0, sticky="ew", padx=(5, 25))
        
        # Configure columns with exact widths
        header_frame.grid_columnconfigure(0, weight=0, minsize=self.COL_WIDTHS["번호"])
        header_frame.grid_columnconfigure(1, weight=0, minsize=self.COL_WIDTHS["지문번호"])
        header_frame.grid_columnconfigure(2, weight=0, minsize=self.COL_WIDTHS["문제유형"])
        header_frame.grid_columnconfigure(3, weight=0, minsize=self.COL_WIDTHS["출제방식"])
        header_frame.grid_columnconfigure(4, weight=0, minsize=self.COL_WIDTHS["수동출제"])

        # Header labels
        CTkLabel(header_frame, width=self.COL_WIDTHS["번호"], height=18, text="번호",
                font=(self.default_font, 11), fg_color="transparent", text_color="black"
                ).grid(row=0, column=0, sticky='nsew', padx=(0, 5), pady=(5, 0))

        CTkLabel(header_frame, width=self.COL_WIDTHS["지문번호"], height=18, text="지문번호",
                font=(self.default_font, 11), fg_color="transparent", text_color="black"
                ).grid(row=0, column=1, sticky='nsew', padx=(5, 5), pady=(5, 0))

        CTkLabel(header_frame, width=self.COL_WIDTHS["문제유형"], height=18, text="문제유형",
                font=(self.default_font, 11), fg_color="transparent", text_color="black"
                ).grid(row=0, column=2, sticky='nsew', padx=(0, 5), pady=(5, 0))

        CTkLabel(header_frame, width=self.COL_WIDTHS["출제방식"], height=18, text="출제방식",
                font=(self.default_font, 11), fg_color="transparent", text_color="black"
                ).grid(row=0, column=3, sticky='nsew', padx=(0, 5), pady=(5, 0))

        CTkLabel(header_frame, width=self.COL_WIDTHS["수동출제"], height=18, text="수동출제 설정",
                font=(self.default_font, 11), fg_color="transparent", text_color="black"
                ).grid(row=0, column=4, sticky='nsew', padx=(0, 5), pady=(5, 0))

        # Scrollable Frame
        self.scrollable_frame = CTkScrollableFrame(main_frame, fg_color="transparent")
        self.scrollable_frame.grid(row=1, column=0, sticky='nsew', padx=0, pady=0)
        
        # Use the same column configurations for scrollable frame
        self.scrollable_frame.grid_columnconfigure(0, weight=0, minsize=self.COL_WIDTHS["번호"])
        self.scrollable_frame.grid_columnconfigure(1, weight=0, minsize=self.COL_WIDTHS["지문번호"])
        self.scrollable_frame.grid_columnconfigure(2, weight=0, minsize=self.COL_WIDTHS["문제유형"])
        self.scrollable_frame.grid_columnconfigure(3, weight=0, minsize=self.COL_WIDTHS["출제방식"])
        self.scrollable_frame.grid_columnconfigure(4, weight=0, minsize=self.COL_WIDTHS["수동출제"])



        # Load all rows
        self.row_widgets = []
        for i, passage_id in enumerate(self.passage_ids, start=1):
            self.add_row(i, passage_id)
            self.update_current_status_label(i, passage_id)            

        

        # Bottom frame with two buttons
        button_frame = CTkFrame(self, fg_color="#2B3D2D")
        button_frame.pack(side='bottom', fill='x', pady=(5, 0))

        # Buttons at the bottom: only "취소" and "출제시작"
        start_button = CTkButton(
            button_frame,
            text="확인",
            width=70,
            height=25,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color='black',
            font=(self.default_font, 13, 'bold'),
            command=self.start
        )
        start_button.pack(side='right', padx=10, pady=5)

        cancel_button = CTkButton(
            button_frame,
            text="기본값 복원",
            width=70,
            height=25,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color='black',
            font=(self.default_font, 13, 'bold'),
            command=self.cancel
        )
        cancel_button.pack(side='left', padx=10, pady=5)

        self.bind("<Escape>", self.cancel)

    def add_row(self, row_idx, passage_id):
        # Column 0: Row number
        index_label = CTkLabel(self.scrollable_frame,
                            text=str(row_idx),
                            font=(self.default_font, 10, "bold"),
                            width=self.COL_WIDTHS["번호"],
                            height=19,
                            corner_radius=5,
                            text_color="black",
                            fg_color="#B6C2B7")
        index_label.grid(row=row_idx, column=0, sticky='ew', padx=(0, 5), pady=3)

        # Column 1: 지문번호
        passage_label = CTkLabel(self.scrollable_frame,
                                text=passage_id,
                                font=(self.default_font, 11),
                                fg_color="#FEF9E0",
                                text_color="black",
                                width=self.COL_WIDTHS["지문번호"]-10,
                                wraplength=self.COL_WIDTHS["지문번호"]-20,
                                anchor="center")
        passage_label.grid(row=row_idx, column=1, sticky='ew', padx=(5, 5), pady=3)

        # Column 2: 문제유형
        type_label = CTkLabel(self.scrollable_frame,
                                text=self.유형,
                                font=(self.default_font, 11),
                                fg_color="#FEF9E0",
                                text_color="black",
                                width=self.COL_WIDTHS["문제유형"],
                                anchor="center")
        type_label.grid(row=row_idx, column=2, sticky='ew', padx=(5, 5), pady=3)

        # Column 3: 출제방식
        current_status_label = CTkLabel(self.scrollable_frame,
                                        font=(self.default_font, 11),
                                        fg_color="#FEF9E0",
                                        width=self.COL_WIDTHS["출제방식"],
                                        anchor="center")

        if self.main_frame_ref and hasattr(self.main_frame_ref, 'manual_edited_texts') and passage_id in self.main_frame_ref.manual_edited_texts:
            current_status_label.configure(text="수동", text_color="red")
        else:
            current_status_label.configure(text="자동", text_color="black")

        current_status_label.grid(row=row_idx, column=3, sticky='ew', padx=(5, 5), pady=3)

        # Column 4: 수동출제 버튼
        manual_button = CTkButton(
            self.scrollable_frame,
            text="수동출제",
            width=self.COL_WIDTHS["수동출제"]-20,  # Slightly smaller to account for padding
            height=25,
            fg_color="white",
            hover_color="#DDA15E",
            text_color="black",
            font=(self.default_font, 11),
            command=lambda idx=row_idx, pid=passage_id: self.on_manual_button_click(idx, pid)
        )
        manual_button.grid(row=row_idx, column=4, sticky='ew', padx=(5, 5), pady=3)

        row_data = {
            'index_label': index_label,
            'passage_label': passage_label,
            'manual_button': manual_button,
            'passage_id': passage_id,
            'current_status_label': current_status_label,
            'row_idx': row_idx
        }
        self.row_widgets.append(row_data)

    def update_current_status_label(self, row_idx, passage_id):
        print("update_current_status_label called")
        print(self.main_frame_ref.manual_edited_texts)
        # Loop through row_widgets to find the matching row
        for row_data in self.row_widgets:
            if row_data['row_idx'] == row_idx:
                current_status_label = row_data['current_status_label']
                # Check if passage_id is now in manual_edited_texts

                수동지문있는지확인용 = passage_id + " // " + self.유형
                print(f"수동지문있는지확인용: {수동지문있는지확인용}")
                if 수동지문있는지확인용 in self.main_frame_ref.manual_edited_texts:
                    current_status_label.configure(text="수동", text_color="red")
                else:
                    current_status_label.configure(text="자동", text_color="black")
                break


    def on_manual_button_click(self, row_idx, passage_id):
        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            self.main_frame_ref.save_manual_placement_text일반(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            # Update the current status label after manual text is saved
            self.update_current_status_label(row_idx, 수동출제할지문번호)

        popup = ManualPlacementPopup(
            parent=self,
            default_font=self.default_font,
            excel_file=self.excel_file,
            row_idx=row_idx,
            수동출제할지문번호=passage_id,
            유형=self.유형,
            난이도="Normal",
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="일반"
        )
        popup.grab_set()



    def start(self):       
        self.destroy()        

    def cancel(self, event=None):       
        self.destroy()        


    def restore_default_values(self, event=None):
        response = messagebox.askyesno("주의", f"수동으로 수정한 지문을 모두 원래 지문으로 복원하시겠습니까?")
        if response is True:

            # Remove all passages shown here from manual_edited_texts
            for row_data in self.row_widgets:
                passage_id = row_data['passage_id']
                if passage_id in self.main_frame_ref.manual_edited_texts:
                    del self.main_frame_ref.manual_edited_texts[passage_id]
                
                # Update the current status label for each row after deletion
                self.update_current_status_label(row_data['row_idx'], passage_id)




class ManualPlacementPopup(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font, excel_file, row_idx, 수동출제할지문번호, 유형, 난이도, save_callback, manual_edited_texts=None, 일반or실모=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title("수동출제할 지문 수정")
        self.geometry("650x400")
        self.minsize(650, 400)
        self.parent = parent  
        self.default_font = default_font
        self.excel_file = excel_file
        self.row_idx = row_idx
        self.수동출제할지문번호 = 수동출제할지문번호
        self.유형 = 유형
        print(f"class ManualPlacementPopup called")
        print(f"유형: {self.유형}")
        self.난이도 = 난이도   
        self.save_callback = save_callback
        self.manual_edited_texts=manual_edited_texts
        print(f"self.manual_edited_texts: {self.manual_edited_texts}")
        self.일반or실모 = 일반or실모
        self.manual_mocktest_popup = manual_mocktest_popup


        self.수동출제할지문번호and유형 = self.수동출제할지문번호 + " // " + self.유형
        self.수동출제할지문번호and유형and난이도 = self.수동출제할지문번호 + " // " + self.유형 + "_" + self.난이도
        
        self.copy_manager = CopyManager(self)

        if self.일반or실모 == "일반":
            if self.수동출제할지문번호and유형 in self.manual_edited_texts:
                self.loaded_text = self.manual_edited_texts[self.수동출제할지문번호and유형]
            else:
                # Load text from Excel
                self.loaded_text = self.load_text_from_excel(self.excel_file, self.수동출제할지문번호)
        elif self.일반or실모 == "실모":
            if self.수동출제할지문번호and유형and난이도 in self.manual_edited_texts:
                self.loaded_text = self.manual_edited_texts[self.수동출제할지문번호and유형and난이도]
            else:
                # Load text from Excel
                self.loaded_text = self.load_text_from_excel(self.excel_file, self.수동출제할지문번호)



        main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        main_frame.pack(fill='both', expand=True, padx=20, pady=20)

        # Row 1: Labels
        label_frame = customtkinter.CTkFrame(main_frame, fg_color="transparent")
        label_frame.pack(side='top', fill='x', pady=(0,10))
        
        row_idx_label0 = customtkinter.CTkLabel(
            label_frame, text="번호:", font=(self.default_font, 12, 'bold'), text_color='black'
        )
        row_idx_label0.pack(side='left', padx=0)
        row_idx_label = customtkinter.CTkEntry(
            label_frame, width=30, border_width=1, corner_radius=0, fg_color="#d7d7d7",
            font=(self.default_font, 12, 'bold'), text_color='black'
        )
        row_idx_label.pack(side='left', padx=(5, 10))
        row_idx_label.insert(0, f"{self.row_idx}")
        row_idx_label.configure(state="disabled")
        

        waiting_label0 = customtkinter.CTkLabel(
            label_frame, text="지문번호:", font=(self.default_font, 12, 'bold'), text_color='black'
        )
        waiting_label0.pack(side='left', padx=0)

        waiting_label = customtkinter.CTkEntry(
            label_frame, width=220, border_width=1, corner_radius=0, fg_color="#d7d7d7",
            font=(self.default_font, 12, 'bold'), text_color='black'
        )
        waiting_label.pack(side='left', padx=(5, 10), fill='x', expand=True)
        waiting_label.insert(0, f"{self.수동출제할지문번호}")
        waiting_label.configure(state="disabled")
        






        난이_label = customtkinter.CTkEntry(
            label_frame, width=60, border_width=1, corner_radius=0, fg_color="#d7d7d7",
            font=(self.default_font, 12, 'bold'), text_color='black')
        난이_label.pack(side='right', padx=(5, 0))
        난이_label.insert(0, f"{self.난이도}")
        난이_label.configure(state="disabled")

        난이_label0 = customtkinter.CTkLabel(
            label_frame, text="난이도:", font=(self.default_font, 12, 'bold'), text_color='black')
        난이_label0.pack(side='right', padx=0)


        유형_label = customtkinter.CTkEntry(
            label_frame, width=80, border_width=1, corner_radius=0, fg_color="#d7d7d7",
            font=(self.default_font, 12, 'bold'), text_color='black')
        유형_label.pack(side='right', padx=(5, 10))
        유형_label.insert(0, f"{self.유형}")
        유형_label.configure(state="disabled")


        유형_label0 = customtkinter.CTkLabel(
            label_frame, text="유형:", font=(self.default_font, 12, 'bold'), text_color='black')
        유형_label0.pack(side='right', padx=0)




        # Row 2: Large Textbox
        self.textbox = customtkinter.CTkTextbox(main_frame, width=550, height=150, wrap='word', fg_color="white", text_color="black", activate_scrollbars=True, undo=True, font=(self.default_font, 14))
        self.textbox.pack(side='top', fill='both', expand=True, pady=(0,10))
        if self.loaded_text:
            self.textbox.insert("1.0", self.loaded_text)


        if self.유형 == "요약":
            self.placeholder_text = "이곳에 요약문을 입력하고, 빈칸으로 출제할 단어 2개를 [[  ]]로 표시하세요."

            self.textbox.configure(state="disabled")
            self.요약문textbox = customtkinter.CTkTextbox(main_frame, width=550, height=50, wrap='word', fg_color="white", text_color="black", activate_scrollbars=True, undo=True, font=(self.default_font, 12))
            self.요약문textbox.pack(side='top', fill='both', expand=True, pady=(0,10))
            self.요약문textbox.insert("0.0", self.placeholder_text)
            self.요약문textbox.configure(text_color='grey')


            # Find the paragraph containing '[[' and assign it to passage
            if self.loaded_text.count('[[') == 2 and self.loaded_text.count(']]') == 2:
                지문passage = ''
                요약문passage = ''
                paragraphs = self.loaded_text.split('\n')
                for paragraph in paragraphs:
                    paragraph = paragraph.strip()  # Remove leading/trailing spaces
                    if not paragraph:  # Skip empty lines
                        continue
                    if '[[' not in paragraph:
                        지문passage += paragraph + '\n'  # Collect all non-summary paragraphs
                    elif '[[' in paragraph:
                        요약문passage = paragraph  # Save the first paragraph with [[ ]]
                        break  # Stop after finding the first 요약문passage

                지문passage = 지문passage.strip()  # Clean up final result
                print(f"지문passage: {지문passage}")
                print(f"요약문passage: {요약문passage}")

                self.textbox.configure(state="normal")
                self.textbox.delete("1.0", "end")
                self.textbox.insert("1.0", 지문passage)
                self.textbox.configure(state="disabled")
                self.요약문textbox.delete("1.0", "end")
                self.요약문textbox.insert("1.0", 요약문passage)
                self.요약문textbox.configure(text_color='black')




            def on_entry_click(event):
                if self.요약문textbox.get("1.0", "end-1c").strip() == self.placeholder_text:
                    self.요약문textbox.delete("1.0", "end")
                    self.요약문textbox.configure(text_color='black')

            def on_entry_focusout(event):
                if not self.요약문textbox.get("1.0", "end-1c").strip():
                    self.요약문textbox.insert("1.0", self.placeholder_text)
                    self.요약문textbox.configure(text_color='grey')

            self.요약문textbox.bind('<FocusIn>', on_entry_click)
            self.요약문textbox.bind('<FocusOut>', on_entry_focusout)

            



        # Row 3: Instruction Label
        if self.유형 == "어휘3단계":
            안내문 = "정답(반의어)으로 출제할 단어 1개에 <   >, 오답(동의어)으로 출제할 단어 4개에 [   ]를 표시하세요.\n출제시 난이도는 Normal를 선택하세요."
        elif self.유형 == "함축의미":
            안내문 = "함축의미 문제를 출제할 어구를 [[   ]]로 표시하세요. (대괄호 2개씩)"
        elif self.유형 == "빈칸추론":
            안내문 = "빈칸으로 출제할 부분을 [[   ]]로 표시하세요. (대괄호 2개씩)"
        elif self.유형 == "순서":
            안내문 = "Normal 유형으로 출제할 경우 3군데, Hard 유형으로 출제할 경우 4군데에 // 를 표시하세요. (한 군데에 슬래시 2개씩)"
        elif self.유형 == "삽입":
            안내문 = "주어진 문장으로 사용할 문장을 [[   ]]로 표시하세요. (대괄호 2개씩)"
        elif self.유형 == "요약":
            안내문 = "아래 창에 요약문을 입력하고, 빈칸으로 출제할 단어 2개를 [[   ]]로 표시하세요. (대괄호 2개씩)"

        elif self.유형 == "영작1":
            안내문 = "영작 문제로 출제할 부분을 [[   ]]로 표시하세요. (대괄호 2개씩)\n(영작1은 단어 배열식입니다.)"
        elif self.유형 == "영작2":
            안내문 = "영작 문제로 출제할 부분을 [[   ]]로 표시하세요. (대괄호 2개씩)\n(영작2는 어형변화, 단어추가 가능 유형입니다.)"



        instruction_label = customtkinter.CTkLabel(
            main_frame,
            text=안내문,
            font=(self.default_font, 11),
            text_color="black"
        )
        instruction_label.pack(side='top', pady=(0,10))

        # Row 4: Buttons
        button_frame = customtkinter.CTkFrame(main_frame, fg_color="transparent")
        button_frame.pack(side='bottom', fill='x', pady=(10,0))


        restore_button = customtkinter.CTkButton(
            button_frame, 
            text="원래 지문 복원",
            font=(self.default_font, 13, 'bold'),
            fg_color="#FEF9E0",
            text_color="black",
            hover_color="#DDA15C",
            command=self.on_restore_default
        )
        restore_button.pack(side='left', padx=5)



        confirm_button = customtkinter.CTkButton(
            button_frame,
            text="확인",
            width=70,
            font=(self.default_font, 13, 'bold'),
            fg_color="#FEF9E0",
            text_color="black",
            hover_color="#DDA15C",
            command=self.on_confirm
        )
        confirm_button.pack(side='right', padx=5)


        cancel_button = customtkinter.CTkButton(
            button_frame, 
            text="닫기",
            width=70,
            font=(self.default_font, 13, 'bold'),
            fg_color="#FEF9E0",
            text_color="black",
            hover_color="#DDA15C",
            command=self.on_cancel
        )
        cancel_button.pack(side='right', padx=5)


        self.modifier_key = None
        if sys.platform.startswith('darwin'):
            # Mac OS
            self.modifier_key = 'Command'
        elif sys.platform.startswith('win'):
            # Windows
            self.modifier_key = 'Control'
        else:
            # Linux or other; adjust as needed
            self.modifier_key = 'Control'


        if sys.platform.startswith('darwin'):                    
            self.textbox.bind("<KeyPress>", self.copy_manager.handle_keypress)
            if self.유형 == "요약":
                self.요약문textbox.bind("<KeyPress>", self.copy_manager.handle_keypress)



        self.bind("<Escape>", self.on_cancel)
        self.textbox.bind(f'<{self.modifier_key}-Return>', self.on_confirm)
        if self.유형 == "요약":
            self.요약문textbox.bind(f'<{self.modifier_key}-Return>', self.on_confirm)


    


    def load_text_from_excel(self, excel_file, waiting_text):
        if not excel_file or not waiting_text:
            return ""

        try:
            wb = load_workbook(excel_file, read_only=True, data_only=True)
            if "대기열" not in wb.sheetnames:
                return ""

            sheet = wb["대기열"]

            # Find the row where column A matches 수동출제할지문번호
            for row in sheet.iter_rows(values_only=True):
                if row and row[0] == waiting_text:
                    # Row found, return column B if available
                    return str(row[1]) if len(row) > 1 and row[1] is not None else ""
            return ""
        except:
            return ""

    def on_restore_default(self, event=None):
        self.loaded_text = self.load_text_from_excel(self.excel_file, self.수동출제할지문번호)
        self.textbox.delete("1.0", "end")
        self.textbox.insert("1.0", self.loaded_text)
        
        if self.일반or실모 == "일반":     
            if self.수동출제할지문번호and유형 in self.manual_edited_texts:
                del self.manual_edited_texts[self.수동출제할지문번호and유형]
        elif self.일반or실모 == "실모":
            if self.수동출제할지문번호and유형and난이도 in self.manual_edited_texts:
                del self.manual_edited_texts[self.수동출제할지문번호and유형and난이도]

        if self.유형 == "요약":
            self.요약문textbox.delete("1.0", "end")
            self.요약문textbox.insert("1.0", self.placeholder_text)
            self.요약문textbox.configure(text_color='grey')

    def on_cancel(self, event=None):
        if self.일반or실모 == "일반":
            if self.parent and self.parent.winfo_exists() and hasattr(self.parent, 'update_current_status_label'):
                self.parent.update_current_status_label(self.row_idx, self.수동출제할지문번호)

        elif self.일반or실모 == "실모":
            if self.manual_mocktest_popup and self.manual_mocktest_popup.winfo_exists() and hasattr(self.manual_mocktest_popup, 'update_current_status_label수동실모'):
                #self.manual_mocktest_popup.update_current_status_label수동실모(self.row_idx, self.수동출제할지문번호and유형and난이도) #하나만
                self.manual_mocktest_popup.update_current_status_label수동실모(None, self.수동출제할지문번호and유형and난이도) #전체


        self.destroy()

    def on_confirm(self, event=None):
        edited_text = self.textbox.get("1.0", "end-1c").strip()

        if self.유형 == "어휘3단계":
            if edited_text.count('[') != 4 and edited_text.count('<') != 1:
                messagebox.showwarning("경고", "정답(반의어)으로 출제할 단어 1개에 <   >, 오답(동의어)으로 출제할 단어 4개에 [   ]를 표시하세요. 출제시 난이도는 Normal를 선택하세요.")
                return

        elif self.유형 == "함축의미":
            if '[[' not in edited_text or ']]' not in edited_text:
                messagebox.showwarning("경고", "함축의미 문제를 출제할 어구를 [[  ]]로 표시하세요.")
                return

        elif self.유형 == "빈칸추론":
            if '[[' not in edited_text or ']]' not in edited_text:
                messagebox.showwarning("경고", "빈칸으로 출제할 부분을 [[  ]]로 표시하세요.")
                return

        elif self.유형 == "순서":
            if edited_text.count('//') != 3 and edited_text.count('//') != 4:
                messagebox.showwarning("경고", "Normal 유형으로 출제할 경우 3군데, Hard 유형으로 출제할 경우 4군데에 // 를 표시하세요. (한 군데에 슬래시 2개씩)")
                return

        elif self.유형 == "삽입":
            if '[[' not in edited_text or ']]' not in edited_text:
                messagebox.showwarning("경고", "주어진 문장으로 사용할 문장을 [[   ]]로 표시하세요. (대괄호 2개씩)")
                return

        elif self.유형 == "요약":
            edited_text요약문 = self.요약문textbox.get("1.0", "end-1c").strip()
            if edited_text요약문 == "" or edited_text요약문.count('[[') != 2 or edited_text요약문.count(']]') != 2:
                messagebox.showwarning("경고", "아래 창에 요약문을 입력하고, 빈칸으로 출제할 단어 2개를 [[  ]]로 표시하세요.")
                return
            edited_text = edited_text + "\n\n" + edited_text요약문



        if self.save_callback:
            self.save_callback(self.row_idx, self.수동출제할지문번호, self.유형, self.난이도, edited_text)
        self.destroy()







class QuestionOptions어휘3단계(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title('어휘3단계 옵션')
        self.geometry('450x100')
        self.minsize(450, 100)

        self.parent = parent
        self.main_frame_ref = main_frame  
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호
        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)


        if 실모flag == True:
            수동출제버튼텍스트 = "어휘3단계(Normal) 수동출제\n(현재 문항)"
            수동출제버튼커맨드 = self.on_manual_placement실모
        else:
            수동출제버튼텍스트 = "어휘3단계(Normal) 수동출제"
            수동출제버튼커맨드 = self.on_manual_placement

        self.manual_button = customtkinter.CTkButton(
            self.main_frame,
            text=수동출제버튼텍스트,
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=수동출제버튼커맨드
        )
        self.manual_button.pack(side='bottom', anchor='center', padx=5, pady=(0, 20))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()



    def on_manual_placement(self):
        
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.destroy()

        popup.grab_set()

    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체
        
        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.destroy()

        popup.grab_set()

        





class QuestionOptions빈칸추론(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None, 검토창에서넘어옴=False):
        super().__init__(parent)
        self.title('빈칸추론 옵션')
        self.geometry('450x350')
        self.minsize(450, 350)

        self.parent = parent
        self.main_frame_ref = main_frame  # Store the reference to main frame
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호
        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup
        self.검토창에서넘어옴 = 검토창에서넘어옴


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Default values
        self.normal_default_min = "1"
        self.normal_default_max = "4"
        self.hard_default_min = "5"
        self.hard_default_max = "10"

        # Values (will be loaded from JSON or default)
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max

        # Mode variable (1 or 2)
        self.빈칸추론_선지재진술_mode = tk.IntVar(value=1)

        # Attempt to load saved values
        self.load_values_from_json()

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)

        # Instruction Label
        intro_label = customtkinter.CTkLabel(
            self.main_frame,
            text="빈칸추론 세부 옵션을 설정합니다. (모든 문항에 적용됩니다)",
            font=(self.default_font, 12),
            text_color="black"
        )
        intro_label.pack(side='top', anchor='center', pady=(0, 10))

        # Frame for mode selection
        mode_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        mode_frame.pack(side='top', fill='x', expand=False, pady=(0, 10))

        mode_label = customtkinter.CTkLabel(
            mode_frame,
            text="정답 재진술:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        mode_label.pack(side='top', anchor='w', pady=(0,5))

        radio_container = customtkinter.CTkFrame(mode_frame, fg_color="transparent")
        radio_container.pack(side='top', fill='x', expand=False)

        self.radio_button_transformed = customtkinter.CTkRadioButton(
            radio_container,
            text='원문을 재진술하여 선지 제작',
            font=(self.default_font, 13),
            variable=self.빈칸추론_선지재진술_mode,
            value=1,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_transformed.grid(row=0, column=0, padx=(0, 10), pady=5, sticky="w")

        self.radio_button_original = customtkinter.CTkRadioButton(
            radio_container,
            text='원문을 그대로 선지 제작',
            font=(self.default_font, 13),
            variable=self.빈칸추론_선지재진술_mode,
            value=2,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_original.grid(row=0, column=1, padx=(10, 10), pady=5, sticky="w")

        # Frame for length configurations
        length_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        length_frame.pack(side='top', fill='x', expand=False, pady=(0,10))

        length_label = customtkinter.CTkLabel(
            length_frame,
            text="선지 길이:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        length_label.pack(side='top', anchor='w', pady=(0))

        # Create a frame using grid for both Normal and Hard settings
        config_frame = customtkinter.CTkFrame(length_frame, fg_color="transparent")
        config_frame.pack(side='top', fill='x', expand=False)

        # Configure columns for uniform resizing
        for col in [1, 3]:
            config_frame.grid_columnconfigure(col, weight=1)

        # Normal row
        normal_label = customtkinter.CTkLabel(
            config_frame,
            text="Normal 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_label.grid(row=0, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_normal.grid(row=0, column=1, padx=(0, 5), pady=5, sticky="we")

        normal_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_tilde.grid(row=0, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_normal.grid(row=0, column=3, padx=(0, 5), pady=5, sticky="we")

        normal_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_unit.grid(row=0, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_btn = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_normal_defaults
        )
        normal_default_btn.grid(row=0, column=5, padx=5, pady=5, sticky="e")

        # Hard row
        hard_label = customtkinter.CTkLabel(
            config_frame,
            text="Hard 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_label.grid(row=1, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_hard.grid(row=1, column=1, padx=(0, 5), pady=5, sticky="we")

        hard_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_tilde.grid(row=1, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_hard.grid(row=1, column=3, padx=(0, 5), pady=5, sticky="we")

        hard_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_unit.grid(row=1, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_hard = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_hard_defaults
        )
        normal_default_hard.grid(row=1, column=5, padx=5, pady=5, sticky="e")

        # Bind focus out events to validate right entries immediately
        self.length_entry2_normal.bind("<FocusOut>", self.validate_normal_range)
        self.length_entry2_hard.bind("<FocusOut>", self.validate_hard_range)

        # Action buttons at the bottom
        action_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        action_frame.pack(side='bottom', fill='x', expand=False, pady=(10,0))


        if not self.검토창에서넘어옴:
            if 실모flag == True:
                수동출제버튼텍스트 = "빈칸 위치 수동출제\n(현재 문항)"
                수동출제버튼커맨드 = self.on_manual_placement실모
            else:
                수동출제버튼텍스트 = "빈칸 위치 수동출제"
                수동출제버튼커맨드 = self.on_manual_placement

            self.manual_button = customtkinter.CTkButton(
                action_frame,
                text=수동출제버튼텍스트,
                font=(self.default_font, 13, 'bold'),
                width=160,
                height=40,
                fg_color="#FEF9E0",
                hover_color="#DDA15C",
                text_color="black",
                command=수동출제버튼커맨드
            )
            self.manual_button.pack(side='left', padx=5, pady=(0, 5))






        self.confirm_button = customtkinter.CTkButton(
            action_frame,
            text="확인",
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=self.on_confirm
        )
        self.confirm_button.pack(side='right', padx=5, pady=(0, 5))

        # Populate entries with loaded or default values
        self.restore_current_values_to_entries()

        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()



    def on_manual_placement(self):
        
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.on_confirm()
        popup.grab_set()


    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체
            #self.manual_mocktest_popup.update_current_status_label수동실모(row_idx, passageid유형난이도합본) #하나만

        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.on_confirm()
        popup.grab_set()



    def on_confirm(self):
        # Validate entries on confirm as well
        normal_valid = self.validate_entries(self.length_entry1_normal, self.length_entry2_normal)
        hard_valid = self.validate_entries(self.length_entry1_hard, self.length_entry2_hard)

        # If validation fails for either Normal or Hard, do not close or save
        if not normal_valid or not hard_valid:
            return

        # If validation passes, proceed
        normal_min_val = self.length_entry1_normal.get().strip()
        normal_max_val = self.length_entry2_normal.get().strip()
        hard_min_val = self.length_entry1_hard.get().strip()
        hard_max_val = self.length_entry2_hard.get().strip()

        # Save to JSON, including mode
        self.save_values_to_json(
            normal_min_val=normal_min_val,
            normal_max_val=normal_max_val,
            hard_min_val=hard_min_val,
            hard_max_val=hard_max_val,
            mode=self.빈칸추론_선지재진술_mode.get()
        )

        self.destroy()

    def validate_entries(self, left_entry, right_entry):
        return self.check_and_fix_range(left_entry, right_entry, show_warning=True)

    def validate_normal_range(self, event=None):
        # Validate immediately after focus leaves the Normal right entry
        self.check_and_fix_range(self.length_entry1_normal, self.length_entry2_normal, show_warning=True)

    def validate_hard_range(self, event=None):
        # Validate immediately after focus leaves the Hard right entry
        self.check_and_fix_range(self.length_entry1_hard, self.length_entry2_hard, show_warning=True)

    def check_and_fix_range(self, left_entry, right_entry, show_warning=False):
        left_val = left_entry.get().strip()
        right_val = right_entry.get().strip()

        # Ensure they are integers
        try:
            left_num = int(left_val)
            right_num = int(right_val)
        except ValueError:
            if show_warning:
                messagebox.showwarning("경고", "숫자를 입력해주세요.")
            return False

        if right_num < left_num:
            if show_warning:
                messagebox.showwarning("경고", f"우측 값이 좌측 값보다 작습니다. 우측 값을 {left_num}으로 설정합니다.")
            right_entry.delete(0, tk.END)
            right_entry.insert(0, str(left_num))
            return False

        return True

    def set_normal_defaults(self):
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.restore_current_values_to_entries()

    def set_hard_defaults(self):
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max
        self.restore_current_values_to_entries()

    def restore_current_values_to_entries(self):
        # Set Normal entries
        self.length_entry1_normal.delete(0, tk.END)
        self.length_entry1_normal.insert(0, self.normal_min_val)
        self.length_entry2_normal.delete(0, tk.END)
        self.length_entry2_normal.insert(0, self.normal_max_val)

        # Set Hard entries
        self.length_entry1_hard.delete(0, tk.END)
        self.length_entry1_hard.insert(0, self.hard_min_val)
        self.length_entry2_hard.delete(0, tk.END)
        self.length_entry2_hard.insert(0, self.hard_max_val)

    def load_values_from_json(self):
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    self.normal_min_val = data.get("빈칸추론normal_min_val", self.normal_default_min)
                    self.normal_max_val = data.get("빈칸추론normal_max_val", self.normal_default_max)
                    self.hard_min_val = data.get("빈칸추론hard_min_val", self.hard_default_min)
                    self.hard_max_val = data.get("빈칸추론hard_max_val", self.hard_default_max)
                    # Load mode if available
                    saved_mode = data.get("빈칸추론mode", 1)
                    self.빈칸추론_선지재진술_mode.set(saved_mode)
            except:
                # If there's an error reading the file, fallback to defaults
                self.normal_min_val = self.normal_default_min
                self.normal_max_val = self.normal_default_max
                self.hard_min_val = self.hard_default_min
                self.hard_max_val = self.hard_default_max
                self.빈칸추론_선지재진술_mode.set(1)
        else:
            # No file, use defaults
            self.normal_min_val = self.normal_default_min
            self.normal_max_val = self.normal_default_max
            self.hard_min_val = self.hard_default_min
            self.hard_max_val = self.hard_default_max
            self.빈칸추론_선지재진술_mode.set(1)

    def save_values_to_json(self, normal_min_val, normal_max_val, hard_min_val, hard_max_val, mode=1):
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir, exist_ok=True)

        data = {
            "빈칸추론normal_min_val": normal_min_val,
            "빈칸추론normal_max_val": normal_max_val,
            "빈칸추론hard_min_val": hard_min_val,
            "빈칸추론hard_max_val": hard_max_val,
            "빈칸추론mode": mode
        }
        with open(self.make_questions_options_file_path, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=4)







class QuestionOptions함축의미(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title('함축의미 옵션')
        print("QuestionOptions함축의미 called (함축의미 옵션)")
        print(f"수동출제할 지문번호: {수동출제할지문번호}")
        self.geometry('450x100')
        self.minsize(450, 100)

        self.parent = parent
        self.main_frame_ref = main_frame  
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호

        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)


        if 실모flag == True:
            수동출제버튼텍스트 = "밑줄 위치 수동출제\n(현재 문항)"
            수동출제버튼커맨드 = self.on_manual_placement실모
        else:
            수동출제버튼텍스트 = "밑줄 위치 수동출제"
            수동출제버튼커맨드 = self.on_manual_placement


        self.manual_button = customtkinter.CTkButton(
            self.main_frame,
            text=수동출제버튼텍스트,
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=수동출제버튼커맨드
        )
        self.manual_button.pack(side='bottom', anchor='center', padx=5, pady=(0, 20))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()


    def on_manual_placement(self):
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.destroy()
        
        popup.grab_set()


    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체

        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.destroy()

        popup.grab_set()

        









class QuestionOptions내용일치(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", main_frame=None):
        super().__init__(parent)
        self.title('내용일치 옵션')
        self.geometry('450x350')
        self.minsize(450, 350)

        self.default_font = default_font

        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Default values
        self.normal_default_min = "10"
        self.normal_default_max = "15"
        self.hard_default_min = "15"
        self.hard_default_max = "20"

        # Values (will be loaded from JSON or default)
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max

        # Mode variable (1 or 2)
        self.내용일치_mode = tk.IntVar(value=1)

        # Attempt to load saved values
        self.load_values_from_json()

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)

        # Instruction Label
        intro_label = customtkinter.CTkLabel(
            self.main_frame,
            text="내용일치 세부 옵션을 설정합니다. (모든 문항에 적용됩니다)",
            font=(self.default_font, 12),
            text_color="black"
        )
        intro_label.pack(side='top', anchor='center', pady=(0, 10))

        """
        # Frame for mode selection
        mode_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        mode_frame.pack(side='top', fill='x', expand=False, pady=(0, 10))

        mode_label = customtkinter.CTkLabel(
            mode_frame,
            text="정답 갯수:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        mode_label.pack(side='top', anchor='w', pady=(0,5))

        radio_container = customtkinter.CTkFrame(mode_frame, fg_color="transparent")
        radio_container.pack(side='top', fill='x', expand=False)

        self.radio_button_multiple = customtkinter.CTkRadioButton(
            radio_container,
            text='정답 여러 개',
            font=(self.default_font, 13),
            variable=self.내용일치_mode,
            value=1,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_multiple.grid(row=0, column=0, padx=(0, 10), pady=5, sticky="w")

        self.radio_button_single = customtkinter.CTkRadioButton(
            radio_container,
            text='정답 1개',
            font=(self.default_font, 13),
            variable=self.내용일치_mode,
            value=2,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_single.grid(row=0, column=1, padx=(10, 10), pady=5, sticky="w")
        """



        # Frame for length configurations
        length_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        length_frame.pack(side='top', fill='x', expand=False, pady=(0,10))

        length_label = customtkinter.CTkLabel(
            length_frame,
            text="선지 길이:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        length_label.pack(side='top', anchor='w', pady=(0))

        # Create a frame using grid for both Normal and Hard settings
        config_frame = customtkinter.CTkFrame(length_frame, fg_color="transparent")
        config_frame.pack(side='top', fill='x', expand=False)

        # Configure columns for uniform resizing
        for col in [1, 3]:
            config_frame.grid_columnconfigure(col, weight=1)

        # Normal row
        normal_label = customtkinter.CTkLabel(
            config_frame,
            text="Normal 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_label.grid(row=0, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_normal.grid(row=0, column=1, padx=(0, 5), pady=5, sticky="we")

        normal_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_tilde.grid(row=0, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_normal.grid(row=0, column=3, padx=(0, 5), pady=5, sticky="we")

        normal_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_unit.grid(row=0, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_btn = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_normal_defaults
        )
        normal_default_btn.grid(row=0, column=5, padx=5, pady=5, sticky="e")

        # Hard row
        hard_label = customtkinter.CTkLabel(
            config_frame,
            text="Hard 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_label.grid(row=1, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_hard.grid(row=1, column=1, padx=(0, 5), pady=5, sticky="we")

        hard_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_tilde.grid(row=1, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_hard.grid(row=1, column=3, padx=(0, 5), pady=5, sticky="we")

        hard_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_unit.grid(row=1, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_hard = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_hard_defaults
        )
        normal_default_hard.grid(row=1, column=5, padx=5, pady=5, sticky="e")

        # Bind focus out events to validate right entries immediately
        self.length_entry2_normal.bind("<FocusOut>", self.validate_normal_range)
        self.length_entry2_hard.bind("<FocusOut>", self.validate_hard_range)

        # Action buttons at the bottom
        action_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        action_frame.pack(side='bottom', fill='x', expand=False, pady=(10,0))

        self.confirm_button = customtkinter.CTkButton(
            action_frame,
            text="확인",
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=self.on_confirm
        )
        self.confirm_button.pack(side='right', padx=5, pady=(0, 5))

        # Populate entries with loaded or default values
        self.restore_current_values_to_entries()

        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()

    def on_confirm(self):
        # Validate entries on confirm as well
        normal_valid = self.validate_entries(self.length_entry1_normal, self.length_entry2_normal)
        hard_valid = self.validate_entries(self.length_entry1_hard, self.length_entry2_hard)

        # If validation fails for either Normal or Hard, do not close or save
        if not normal_valid or not hard_valid:
            return

        # If validation passes, proceed
        normal_min_val = self.length_entry1_normal.get().strip()
        normal_max_val = self.length_entry2_normal.get().strip()
        hard_min_val = self.length_entry1_hard.get().strip()
        hard_max_val = self.length_entry2_hard.get().strip()

        # Save to JSON, including mode
        self.save_values_to_json(
            normal_min_val=normal_min_val,
            normal_max_val=normal_max_val,
            hard_min_val=hard_min_val,
            hard_max_val=hard_max_val,
            mode=self.내용일치_mode.get()
        )

        self.destroy()

    def validate_entries(self, left_entry, right_entry):
        return self.check_and_fix_range(left_entry, right_entry, show_warning=True)

    def validate_normal_range(self, event=None):
        # Validate immediately after focus leaves the Normal right entry
        self.check_and_fix_range(self.length_entry1_normal, self.length_entry2_normal, show_warning=True)

    def validate_hard_range(self, event=None):
        # Validate immediately after focus leaves the Hard right entry
        self.check_and_fix_range(self.length_entry1_hard, self.length_entry2_hard, show_warning=True)

    def check_and_fix_range(self, left_entry, right_entry, show_warning=False):
        left_val = left_entry.get().strip()
        right_val = right_entry.get().strip()

        # Ensure they are integers
        try:
            left_num = int(left_val)
            right_num = int(right_val)
        except ValueError:
            if show_warning:
                messagebox.showwarning("경고", "숫자를 입력해주세요.")
            return False

        if right_num < left_num:
            if show_warning:
                messagebox.showwarning("경고", f"우측 값이 좌측 값보다 작습니다. 우측 값을 {left_num}으로 설정합니다.")
            right_entry.delete(0, tk.END)
            right_entry.insert(0, str(left_num))
            return False

        return True

    def set_normal_defaults(self):
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.restore_current_values_to_entries()

    def set_hard_defaults(self):
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max
        self.restore_current_values_to_entries()

    def restore_current_values_to_entries(self):
        # Set Normal entries
        self.length_entry1_normal.delete(0, tk.END)
        self.length_entry1_normal.insert(0, self.normal_min_val)
        self.length_entry2_normal.delete(0, tk.END)
        self.length_entry2_normal.insert(0, self.normal_max_val)

        # Set Hard entries
        self.length_entry1_hard.delete(0, tk.END)
        self.length_entry1_hard.insert(0, self.hard_min_val)
        self.length_entry2_hard.delete(0, tk.END)
        self.length_entry2_hard.insert(0, self.hard_max_val)

    def load_values_from_json(self):
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    self.normal_min_val = data.get("내용일치normal_min_val", self.normal_default_min)
                    self.normal_max_val = data.get("내용일치normal_max_val", self.normal_default_max)
                    self.hard_min_val = data.get("내용일치hard_min_val", self.hard_default_min)
                    self.hard_max_val = data.get("내용일치hard_max_val", self.hard_default_max)
                    saved_mode = data.get("내용일치mode", 1)
                    self.내용일치_mode.set(saved_mode)
            except:
                # If there's an error reading the file, fallback to defaults
                self.normal_min_val = self.normal_default_min
                self.normal_max_val = self.normal_default_max
                self.hard_min_val = self.hard_default_min
                self.hard_max_val = self.hard_default_max
                self.내용일치_mode.set(1)
        else:
            # No file, use defaults
            self.normal_min_val = self.normal_default_min
            self.normal_max_val = self.normal_default_max
            self.hard_min_val = self.hard_default_min
            self.hard_max_val = self.hard_default_max
            self.내용일치_mode.set(1)

    def save_values_to_json(self, normal_min_val, normal_max_val, hard_min_val, hard_max_val, mode=1):
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir, exist_ok=True)

        # Load existing data if any
        existing_data = {}
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    existing_data = json.load(f)
            except:
                existing_data = {}

        # Update data with new values
        existing_data["내용일치normal_min_val"] = normal_min_val
        existing_data["내용일치normal_max_val"] = normal_max_val
        existing_data["내용일치hard_min_val"] = hard_min_val
        existing_data["내용일치hard_max_val"] = hard_max_val
        existing_data["내용일치mode"] = mode

        with open(self.make_questions_options_file_path, 'w', encoding='utf-8') as f:
            json.dump(existing_data, f, ensure_ascii=False, indent=4)










class QuestionOptions내용일치한(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", main_frame=None):
        super().__init__(parent)
        self.title('내용일치(한) 옵션')
        self.geometry('450x350')
        self.minsize(450, 350)

        self.default_font = default_font

        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Default values
        self.normal_default_min = "10"
        self.normal_default_max = "20"
        self.hard_default_min = "15"
        self.hard_default_max = "25"

        # Values (will be loaded from JSON or default)
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max

        # Mode variable (1 or 2)
        self.내용일치_mode = tk.IntVar(value=1)

        # Attempt to load saved values
        self.load_values_from_json()

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)

        # Instruction Label
        intro_label = customtkinter.CTkLabel(
            self.main_frame,
            text="내용일치(한) 세부 옵션을 설정합니다. (모든 문항에 적용됩니다)",
            font=(self.default_font, 12),
            text_color="black"
        )
        intro_label.pack(side='top', anchor='center', pady=(0, 10))



        # Frame for length configurations
        length_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        length_frame.pack(side='top', fill='x', expand=False, pady=(0,10))

        length_label = customtkinter.CTkLabel(
            length_frame,
            text="선지 길이:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        length_label.pack(side='top', anchor='w', pady=(0))

        # Create a frame using grid for both Normal and Hard settings
        config_frame = customtkinter.CTkFrame(length_frame, fg_color="transparent")
        config_frame.pack(side='top', fill='x', expand=False)

        # Configure columns for uniform resizing
        for col in [1, 3]:
            config_frame.grid_columnconfigure(col, weight=1)

        # Normal row
        normal_label = customtkinter.CTkLabel(
            config_frame,
            text="Normal 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_label.grid(row=0, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_normal.grid(row=0, column=1, padx=(0, 5), pady=5, sticky="we")

        normal_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_tilde.grid(row=0, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_normal.grid(row=0, column=3, padx=(0, 5), pady=5, sticky="we")

        normal_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_unit.grid(row=0, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_btn = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_normal_defaults
        )
        normal_default_btn.grid(row=0, column=5, padx=5, pady=5, sticky="e")

        # Hard row
        hard_label = customtkinter.CTkLabel(
            config_frame,
            text="Hard 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_label.grid(row=1, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_hard.grid(row=1, column=1, padx=(0, 5), pady=5, sticky="we")

        hard_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_tilde.grid(row=1, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_hard.grid(row=1, column=3, padx=(0, 5), pady=5, sticky="we")

        hard_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_unit.grid(row=1, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_hard = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_hard_defaults
        )
        normal_default_hard.grid(row=1, column=5, padx=5, pady=5, sticky="e")

        # Bind focus out events to validate right entries immediately
        self.length_entry2_normal.bind("<FocusOut>", self.validate_normal_range)
        self.length_entry2_hard.bind("<FocusOut>", self.validate_hard_range)

        # Action buttons at the bottom
        action_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        action_frame.pack(side='bottom', fill='x', expand=False, pady=(10,0))

        self.confirm_button = customtkinter.CTkButton(
            action_frame,
            text="확인",
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=self.on_confirm
        )
        self.confirm_button.pack(side='right', padx=5, pady=(0, 5))

        # Populate entries with loaded or default values
        self.restore_current_values_to_entries()

        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()

    def on_confirm(self):
        # Validate entries on confirm as well
        normal_valid = self.validate_entries(self.length_entry1_normal, self.length_entry2_normal)
        hard_valid = self.validate_entries(self.length_entry1_hard, self.length_entry2_hard)

        # If validation fails for either Normal or Hard, do not close or save
        if not normal_valid or not hard_valid:
            return

        # If validation passes, proceed
        normal_min_val = self.length_entry1_normal.get().strip()
        normal_max_val = self.length_entry2_normal.get().strip()
        hard_min_val = self.length_entry1_hard.get().strip()
        hard_max_val = self.length_entry2_hard.get().strip()

        # Save to JSON, including mode
        self.save_values_to_json(
            normal_min_val=normal_min_val,
            normal_max_val=normal_max_val,
            hard_min_val=hard_min_val,
            hard_max_val=hard_max_val,
            mode=self.내용일치_mode.get()
        )

        self.destroy()

    def validate_entries(self, left_entry, right_entry):
        return self.check_and_fix_range(left_entry, right_entry, show_warning=True)

    def validate_normal_range(self, event=None):
        # Validate immediately after focus leaves the Normal right entry
        self.check_and_fix_range(self.length_entry1_normal, self.length_entry2_normal, show_warning=True)

    def validate_hard_range(self, event=None):
        # Validate immediately after focus leaves the Hard right entry
        self.check_and_fix_range(self.length_entry1_hard, self.length_entry2_hard, show_warning=True)

    def check_and_fix_range(self, left_entry, right_entry, show_warning=False):
        left_val = left_entry.get().strip()
        right_val = right_entry.get().strip()

        # Ensure they are integers
        try:
            left_num = int(left_val)
            right_num = int(right_val)
        except ValueError:
            if show_warning:
                messagebox.showwarning("경고", "숫자를 입력해주세요.")
            return False

        if right_num < left_num:
            if show_warning:
                messagebox.showwarning("경고", f"우측 값이 좌측 값보다 작습니다. 우측 값을 {left_num}으로 설정합니다.")
            right_entry.delete(0, tk.END)
            right_entry.insert(0, str(left_num))
            return False

        return True

    def set_normal_defaults(self):
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.restore_current_values_to_entries()

    def set_hard_defaults(self):
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max
        self.restore_current_values_to_entries()

    def restore_current_values_to_entries(self):
        # Set Normal entries
        self.length_entry1_normal.delete(0, tk.END)
        self.length_entry1_normal.insert(0, self.normal_min_val)
        self.length_entry2_normal.delete(0, tk.END)
        self.length_entry2_normal.insert(0, self.normal_max_val)

        # Set Hard entries
        self.length_entry1_hard.delete(0, tk.END)
        self.length_entry1_hard.insert(0, self.hard_min_val)
        self.length_entry2_hard.delete(0, tk.END)
        self.length_entry2_hard.insert(0, self.hard_max_val)

    def load_values_from_json(self):
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    self.normal_min_val = data.get("내용일치normal_min_val", self.normal_default_min)
                    self.normal_max_val = data.get("내용일치normal_max_val", self.normal_default_max)
                    self.hard_min_val = data.get("내용일치hard_min_val", self.hard_default_min)
                    self.hard_max_val = data.get("내용일치hard_max_val", self.hard_default_max)
                    saved_mode = data.get("내용일치mode", 1)
                    self.내용일치_mode.set(saved_mode)
            except:
                # If there's an error reading the file, fallback to defaults
                self.normal_min_val = self.normal_default_min
                self.normal_max_val = self.normal_default_max
                self.hard_min_val = self.hard_default_min
                self.hard_max_val = self.hard_default_max
                self.내용일치_mode.set(1)
        else:
            # No file, use defaults
            self.normal_min_val = self.normal_default_min
            self.normal_max_val = self.normal_default_max
            self.hard_min_val = self.hard_default_min
            self.hard_max_val = self.hard_default_max
            self.내용일치_mode.set(1)

    def save_values_to_json(self, normal_min_val, normal_max_val, hard_min_val, hard_max_val, mode=1):
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir, exist_ok=True)

        # Load existing data if any
        existing_data = {}
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    existing_data = json.load(f)
            except:
                existing_data = {}

        # Update data with new values
        existing_data["내용일치normal_min_val"] = normal_min_val
        existing_data["내용일치normal_max_val"] = normal_max_val
        existing_data["내용일치hard_min_val"] = hard_min_val
        existing_data["내용일치hard_max_val"] = hard_max_val
        existing_data["내용일치mode"] = mode

        with open(self.make_questions_options_file_path, 'w', encoding='utf-8') as f:
            json.dump(existing_data, f, ensure_ascii=False, indent=4)










class QuestionOptions추론(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", main_frame=None):
        super().__init__(parent)
        self.title('추론 옵션')
        self.geometry('450x350')
        self.minsize(450, 350)

        self.default_font = default_font

        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Default values
        self.normal_default_min = "10"
        self.normal_default_max = "15"
        self.hard_default_min = "17"
        self.hard_default_max = "22"

        # Values (will be loaded from JSON or default)
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max

        # Mode variable (1 or 2)
        self.추론_mode = tk.IntVar(value=1)

        # Attempt to load saved values
        self.load_values_from_json()

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)

        # Instruction Label
        intro_label = customtkinter.CTkLabel(
            self.main_frame,
            text="추론 세부 옵션을 설정합니다. (모든 문항에 적용됩니다)",
            font=(self.default_font, 12),
            text_color="black"
        )
        intro_label.pack(side='top', anchor='center', pady=(0, 10))

        # Frame for mode selection
        mode_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        mode_frame.pack(side='top', fill='x', expand=False, pady=(0, 10))

        mode_label = customtkinter.CTkLabel(
            mode_frame,
            text="정답 갯수:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        mode_label.pack(side='top', anchor='w', pady=(0,5))

        radio_container = customtkinter.CTkFrame(mode_frame, fg_color="transparent")
        radio_container.pack(side='top', fill='x', expand=False)

        self.radio_button_multiple = customtkinter.CTkRadioButton(
            radio_container,
            text='정답 1개',
            font=(self.default_font, 13),
            variable=self.추론_mode,
            value=1,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_multiple.grid(row=0, column=0, padx=(0, 10), pady=5, sticky="w")

        self.radio_button_single = customtkinter.CTkRadioButton(
            radio_container,
            text='정답 2개',
            font=(self.default_font, 13),
            variable=self.추론_mode,
            value=2,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_single.grid(row=0, column=1, padx=(10, 10), pady=5, sticky="w")

        # Frame for length configurations
        length_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        length_frame.pack(side='top', fill='x', expand=False, pady=(0,10))

        length_label = customtkinter.CTkLabel(
            length_frame,
            text="선지 길이:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        length_label.pack(side='top', anchor='w', pady=(0))

        # Create a frame using grid for both Normal and Hard settings
        config_frame = customtkinter.CTkFrame(length_frame, fg_color="transparent")
        config_frame.pack(side='top', fill='x', expand=False)

        # Configure columns for uniform resizing
        for col in [1, 3]:
            config_frame.grid_columnconfigure(col, weight=1)

        # Normal row
        normal_label = customtkinter.CTkLabel(
            config_frame,
            text="Normal 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_label.grid(row=0, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_normal.grid(row=0, column=1, padx=(0, 5), pady=5, sticky="we")

        normal_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_tilde.grid(row=0, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_normal.grid(row=0, column=3, padx=(0, 5), pady=5, sticky="we")

        normal_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_unit.grid(row=0, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_btn = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_normal_defaults
        )
        normal_default_btn.grid(row=0, column=5, padx=5, pady=5, sticky="e")

        # Hard row
        hard_label = customtkinter.CTkLabel(
            config_frame,
            text="Hard 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_label.grid(row=1, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_hard.grid(row=1, column=1, padx=(0, 5), pady=5, sticky="we")

        hard_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_tilde.grid(row=1, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_hard.grid(row=1, column=3, padx=(0, 5), pady=5, sticky="we")

        hard_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_unit.grid(row=1, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_hard = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_hard_defaults
        )
        normal_default_hard.grid(row=1, column=5, padx=5, pady=5, sticky="e")

        # Bind focus out events to validate right entries immediately
        self.length_entry2_normal.bind("<FocusOut>", self.validate_normal_range)
        self.length_entry2_hard.bind("<FocusOut>", self.validate_hard_range)

        # Action buttons at the bottom
        action_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        action_frame.pack(side='bottom', fill='x', expand=False, pady=(10,0))

        self.confirm_button = customtkinter.CTkButton(
            action_frame,
            text="확인",
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=self.on_confirm
        )
        self.confirm_button.pack(side='right', padx=5, pady=(0, 5))

        # Populate entries with loaded or default values
        self.restore_current_values_to_entries()

        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()

    def on_confirm(self):
        # Validate entries on confirm as well
        normal_valid = self.validate_entries(self.length_entry1_normal, self.length_entry2_normal)
        hard_valid = self.validate_entries(self.length_entry1_hard, self.length_entry2_hard)

        # If validation fails for either Normal or Hard, do not close or save
        if not normal_valid or not hard_valid:
            return

        # If validation passes, proceed
        normal_min_val = self.length_entry1_normal.get().strip()
        normal_max_val = self.length_entry2_normal.get().strip()
        hard_min_val = self.length_entry1_hard.get().strip()
        hard_max_val = self.length_entry2_hard.get().strip()

        # Save to JSON, including mode
        self.save_values_to_json(
            normal_min_val=normal_min_val,
            normal_max_val=normal_max_val,
            hard_min_val=hard_min_val,
            hard_max_val=hard_max_val,
            mode=self.추론_mode.get()
        )

        self.destroy()

    def validate_entries(self, left_entry, right_entry):
        return self.check_and_fix_range(left_entry, right_entry, show_warning=True)

    def validate_normal_range(self, event=None):
        # Validate immediately after focus leaves the Normal right entry
        self.check_and_fix_range(self.length_entry1_normal, self.length_entry2_normal, show_warning=True)

    def validate_hard_range(self, event=None):
        # Validate immediately after focus leaves the Hard right entry
        self.check_and_fix_range(self.length_entry1_hard, self.length_entry2_hard, show_warning=True)

    def check_and_fix_range(self, left_entry, right_entry, show_warning=False):
        left_val = left_entry.get().strip()
        right_val = right_entry.get().strip()

        # Ensure they are integers
        try:
            left_num = int(left_val)
            right_num = int(right_val)
        except ValueError:
            if show_warning:
                messagebox.showwarning("경고", "숫자를 입력해주세요.")
            return False

        if right_num < left_num:
            if show_warning:
                messagebox.showwarning("경고", f"우측 값이 좌측 값보다 작습니다. 우측 값을 {left_num}으로 설정합니다.")
            right_entry.delete(0, tk.END)
            right_entry.insert(0, str(left_num))
            return False

        return True

    def set_normal_defaults(self):
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.restore_current_values_to_entries()

    def set_hard_defaults(self):
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max
        self.restore_current_values_to_entries()

    def restore_current_values_to_entries(self):
        # Set Normal entries
        self.length_entry1_normal.delete(0, tk.END)
        self.length_entry1_normal.insert(0, self.normal_min_val)
        self.length_entry2_normal.delete(0, tk.END)
        self.length_entry2_normal.insert(0, self.normal_max_val)

        # Set Hard entries
        self.length_entry1_hard.delete(0, tk.END)
        self.length_entry1_hard.insert(0, self.hard_min_val)
        self.length_entry2_hard.delete(0, tk.END)
        self.length_entry2_hard.insert(0, self.hard_max_val)

    def load_values_from_json(self):
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    self.normal_min_val = data.get("추론normal_min_val", self.normal_default_min)
                    self.normal_max_val = data.get("추론normal_max_val", self.normal_default_max)
                    self.hard_min_val = data.get("추론hard_min_val", self.hard_default_min)
                    self.hard_max_val = data.get("추론hard_max_val", self.hard_default_max)
                    saved_mode = data.get("추론mode", 1)
                    self.추론_mode.set(saved_mode)
            except:
                # If there's an error reading the file, fallback to defaults
                self.normal_min_val = self.normal_default_min
                self.normal_max_val = self.normal_default_max
                self.hard_min_val = self.hard_default_min
                self.hard_max_val = self.hard_default_max
                self.추론_mode.set(1)
        else:
            # No file, use defaults
            self.normal_min_val = self.normal_default_min
            self.normal_max_val = self.normal_default_max
            self.hard_min_val = self.hard_default_min
            self.hard_max_val = self.hard_default_max
            self.추론_mode.set(1)

    def save_values_to_json(self, normal_min_val, normal_max_val, hard_min_val, hard_max_val, mode=1):
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir, exist_ok=True)

        # Load existing data if any
        existing_data = {}
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    existing_data = json.load(f)
            except:
                existing_data = {}

        # Update data with new values
        existing_data["추론normal_min_val"] = normal_min_val
        existing_data["추론normal_max_val"] = normal_max_val
        existing_data["추론hard_min_val"] = hard_min_val
        existing_data["추론hard_max_val"] = hard_max_val
        existing_data["추론mode"] = mode

        with open(self.make_questions_options_file_path, 'w', encoding='utf-8') as f:
            json.dump(existing_data, f, ensure_ascii=False, indent=4)








class QuestionOptions추론불가(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", main_frame=None):
        super().__init__(parent)
        self.title('추론불가 옵션')
        self.geometry('450x350')
        self.minsize(450, 350)

        self.default_font = default_font

        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Default values
        self.normal_default_min = "10"
        self.normal_default_max = "15"
        self.hard_default_min = "17"
        self.hard_default_max = "22"

        # Values (will be loaded from JSON or default)
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max

        # Mode variable (1 or 2)
        self.추론불가_mode = tk.IntVar(value=1)

        # Attempt to load saved values
        self.load_values_from_json()

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)

        # Instruction Label
        intro_label = customtkinter.CTkLabel(
            self.main_frame,
            text="추론불가 세부 옵션을 설정합니다. (모든 문항에 적용됩니다)",
            font=(self.default_font, 12),
            text_color="black"
        )
        intro_label.pack(side='top', anchor='center', pady=(0, 10))

        # Frame for mode selection
        mode_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        mode_frame.pack(side='top', fill='x', expand=False, pady=(0, 10))

        mode_label = customtkinter.CTkLabel(
            mode_frame,
            text="정답 갯수:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        mode_label.pack(side='top', anchor='w', pady=(0,5))

        radio_container = customtkinter.CTkFrame(mode_frame, fg_color="transparent")
        radio_container.pack(side='top', fill='x', expand=False)

        self.radio_button_multiple = customtkinter.CTkRadioButton(
            radio_container,
            text='정답 1개',
            font=(self.default_font, 13),
            variable=self.추론불가_mode,
            value=1,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_multiple.grid(row=0, column=0, padx=(0, 10), pady=5, sticky="w")

        self.radio_button_single = customtkinter.CTkRadioButton(
            radio_container,
            text='정답 2개',
            font=(self.default_font, 13),
            variable=self.추론불가_mode,
            value=2,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_single.grid(row=0, column=1, padx=(10, 10), pady=5, sticky="w")

        # Frame for length configurations
        length_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        length_frame.pack(side='top', fill='x', expand=False, pady=(0,10))

        length_label = customtkinter.CTkLabel(
            length_frame,
            text="선지 길이:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        length_label.pack(side='top', anchor='w', pady=(0))

        # Create a frame using grid for both Normal and Hard settings
        config_frame = customtkinter.CTkFrame(length_frame, fg_color="transparent")
        config_frame.pack(side='top', fill='x', expand=False)

        # Configure columns for uniform resizing
        for col in [1, 3]:
            config_frame.grid_columnconfigure(col, weight=1)

        # Normal row
        normal_label = customtkinter.CTkLabel(
            config_frame,
            text="Normal 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_label.grid(row=0, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_normal.grid(row=0, column=1, padx=(0, 5), pady=5, sticky="we")

        normal_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_tilde.grid(row=0, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_normal = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_normal.grid(row=0, column=3, padx=(0, 5), pady=5, sticky="we")

        normal_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        normal_unit.grid(row=0, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_btn = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_normal_defaults
        )
        normal_default_btn.grid(row=0, column=5, padx=5, pady=5, sticky="e")

        # Hard row
        hard_label = customtkinter.CTkLabel(
            config_frame,
            text="Hard 선지 길이:",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_label.grid(row=1, column=0, padx=(0, 5), pady=5, sticky="e")

        self.length_entry1_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry1_hard.grid(row=1, column=1, padx=(0, 5), pady=5, sticky="we")

        hard_tilde = customtkinter.CTkLabel(
            config_frame,
            text=" ~ ",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_tilde.grid(row=1, column=2, padx=(5, 5), pady=5, sticky="we")

        self.length_entry2_hard = customtkinter.CTkEntry(
            config_frame,
            width=30,
            font=(self.default_font, 13),
            fg_color="white",
            text_color="black"
        )
        self.length_entry2_hard.grid(row=1, column=3, padx=(0, 5), pady=5, sticky="we")

        hard_unit = customtkinter.CTkLabel(
            config_frame,
            text="단어",
            font=(self.default_font, 13),
            text_color="black"
        )
        hard_unit.grid(row=1, column=4, padx=(5, 5), pady=5, sticky="w")

        normal_default_hard = customtkinter.CTkButton(
            config_frame,
            text="기본값",
            font=(self.default_font, 12, 'bold'),
            width=40,
            height=25,
            fg_color="white",
            hover_color="#DDA15C",
            text_color="black",
            command=self.set_hard_defaults
        )
        normal_default_hard.grid(row=1, column=5, padx=5, pady=5, sticky="e")

        # Bind focus out events to validate right entries immediately
        self.length_entry2_normal.bind("<FocusOut>", self.validate_normal_range)
        self.length_entry2_hard.bind("<FocusOut>", self.validate_hard_range)

        # Action buttons at the bottom
        action_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        action_frame.pack(side='bottom', fill='x', expand=False, pady=(10,0))

        self.confirm_button = customtkinter.CTkButton(
            action_frame,
            text="확인",
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=self.on_confirm
        )
        self.confirm_button.pack(side='right', padx=5, pady=(0, 5))

        # Populate entries with loaded or default values
        self.restore_current_values_to_entries()

        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()

    def on_confirm(self):
        # Validate entries on confirm as well
        normal_valid = self.validate_entries(self.length_entry1_normal, self.length_entry2_normal)
        hard_valid = self.validate_entries(self.length_entry1_hard, self.length_entry2_hard)

        # If validation fails for either Normal or Hard, do not close or save
        if not normal_valid or not hard_valid:
            return

        # If validation passes, proceed
        normal_min_val = self.length_entry1_normal.get().strip()
        normal_max_val = self.length_entry2_normal.get().strip()
        hard_min_val = self.length_entry1_hard.get().strip()
        hard_max_val = self.length_entry2_hard.get().strip()

        # Save to JSON, including mode
        self.save_values_to_json(
            normal_min_val=normal_min_val,
            normal_max_val=normal_max_val,
            hard_min_val=hard_min_val,
            hard_max_val=hard_max_val,
            mode=self.추론불가_mode.get()
        )

        self.destroy()

    def validate_entries(self, left_entry, right_entry):
        return self.check_and_fix_range(left_entry, right_entry, show_warning=True)

    def validate_normal_range(self, event=None):
        # Validate immediately after focus leaves the Normal right entry
        self.check_and_fix_range(self.length_entry1_normal, self.length_entry2_normal, show_warning=True)

    def validate_hard_range(self, event=None):
        # Validate immediately after focus leaves the Hard right entry
        self.check_and_fix_range(self.length_entry1_hard, self.length_entry2_hard, show_warning=True)

    def check_and_fix_range(self, left_entry, right_entry, show_warning=False):
        left_val = left_entry.get().strip()
        right_val = right_entry.get().strip()

        # Ensure they are integers
        try:
            left_num = int(left_val)
            right_num = int(right_val)
        except ValueError:
            if show_warning:
                messagebox.showwarning("경고", "숫자를 입력해주세요.")
            return False

        if right_num < left_num:
            if show_warning:
                messagebox.showwarning("경고", f"우측 값이 좌측 값보다 작습니다. 우측 값을 {left_num}으로 설정합니다.")
            right_entry.delete(0, tk.END)
            right_entry.insert(0, str(left_num))
            return False

        return True

    def set_normal_defaults(self):
        self.normal_min_val = self.normal_default_min
        self.normal_max_val = self.normal_default_max
        self.restore_current_values_to_entries()

    def set_hard_defaults(self):
        self.hard_min_val = self.hard_default_min
        self.hard_max_val = self.hard_default_max
        self.restore_current_values_to_entries()

    def restore_current_values_to_entries(self):
        # Set Normal entries
        self.length_entry1_normal.delete(0, tk.END)
        self.length_entry1_normal.insert(0, self.normal_min_val)
        self.length_entry2_normal.delete(0, tk.END)
        self.length_entry2_normal.insert(0, self.normal_max_val)

        # Set Hard entries
        self.length_entry1_hard.delete(0, tk.END)
        self.length_entry1_hard.insert(0, self.hard_min_val)
        self.length_entry2_hard.delete(0, tk.END)
        self.length_entry2_hard.insert(0, self.hard_max_val)

    def load_values_from_json(self):
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    data = json.load(f)
                    self.normal_min_val = data.get("추론불가normal_min_val", self.normal_default_min)
                    self.normal_max_val = data.get("추론불가normal_max_val", self.normal_default_max)
                    self.hard_min_val = data.get("추론불가hard_min_val", self.hard_default_min)
                    self.hard_max_val = data.get("추론불가hard_max_val", self.hard_default_max)
                    saved_mode = data.get("추론불가mode", 1)
                    self.추론불가_mode.set(saved_mode)
            except:
                # If there's an error reading the file, fallback to defaults
                self.normal_min_val = self.normal_default_min
                self.normal_max_val = self.normal_default_max
                self.hard_min_val = self.hard_default_min
                self.hard_max_val = self.hard_default_max
                self.추론불가_mode.set(1)
        else:
            # No file, use defaults
            self.normal_min_val = self.normal_default_min
            self.normal_max_val = self.normal_default_max
            self.hard_min_val = self.hard_default_min
            self.hard_max_val = self.hard_default_max
            self.추론불가_mode.set(1)

    def save_values_to_json(self, normal_min_val, normal_max_val, hard_min_val, hard_max_val, mode=1):
        print("save_values_to_json called")
        print(f"mode: {mode}")
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir, exist_ok=True)

        # Load existing data if any
        existing_data = {}
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    existing_data = json.load(f)
            except:
                existing_data = {}

        # Update data with new values
        existing_data["추론불가normal_min_val"] = normal_min_val
        existing_data["추론불가normal_max_val"] = normal_max_val
        existing_data["추론불가hard_min_val"] = hard_min_val
        existing_data["추론불가hard_max_val"] = hard_max_val
        existing_data["추론불가mode"] = mode

        with open(self.make_questions_options_file_path, 'w', encoding='utf-8') as f:
            json.dump(existing_data, f, ensure_ascii=False, indent=4)








class QuestionOptions순서(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title('순서 옵션')
        self.geometry('450x100')
        self.minsize(450, 100)

        self.parent = parent
        self.main_frame_ref = main_frame  
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호
        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)


        if 실모flag == True:
            수동출제버튼텍스트 = "순서 수동출제\n(현재 문항)"
            수동출제버튼커맨드 = self.on_manual_placement실모
        else:
            수동출제버튼텍스트 = "순서 수동출제"
            수동출제버튼커맨드 = self.on_manual_placement

        self.manual_button = customtkinter.CTkButton(
            self.main_frame,
            text=수동출제버튼텍스트,
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=수동출제버튼커맨드
        )
        self.manual_button.pack(side='bottom', anchor='center', padx=5, pady=(0, 20))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()



    def on_manual_placement(self):
        
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.destroy()

        popup.grab_set()

    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체
        
        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.destroy()

        popup.grab_set()

        








class QuestionOptions삽입(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title('삽입 옵션')
        self.geometry('450x100')
        self.minsize(450, 100)

        self.parent = parent
        self.main_frame_ref = main_frame  
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호
        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)


        if 실모flag == True:
            수동출제버튼텍스트 = "삽입 수동출제\n(현재 문항)"
            수동출제버튼커맨드 = self.on_manual_placement실모
        else:
            수동출제버튼텍스트 = "삽입 수동출제"
            수동출제버튼커맨드 = self.on_manual_placement

        self.manual_button = customtkinter.CTkButton(
            self.main_frame,
            text=수동출제버튼텍스트,
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=수동출제버튼커맨드
        )
        self.manual_button.pack(side='bottom', anchor='center', padx=5, pady=(0, 20))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()



    def on_manual_placement(self):
        
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.destroy()

        popup.grab_set()

    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체
        
        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.destroy()

        popup.grab_set()

        







class QuestionOptions요지(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", main_frame=None):
        super().__init__(parent)
        self.title('요지 옵션')
        self.geometry('450x350')
        self.minsize(450, 350)

        self.default_font = default_font

        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")


        # Mode variable (1 or 2)
        self.요지_mode = tk.IntVar(value=1)

        # Attempt to load saved values
        self.load_values_from_json()

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)

        # Instruction Label
        intro_label = customtkinter.CTkLabel(
            self.main_frame,
            text="요지 세부 옵션을 설정합니다. (모든 문항에 적용됩니다)",
            font=(self.default_font, 12),
            text_color="black"
        )
        intro_label.pack(side='top', anchor='center', pady=(0, 10))

        # Frame for mode selection
        mode_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        mode_frame.pack(side='top', fill='x', expand=False, pady=(0, 10))

        mode_label = customtkinter.CTkLabel(
            mode_frame,
            text="선지 언어:",
            font=(self.default_font, 13, 'bold'),
            text_color="black"
        )
        mode_label.pack(side='top', anchor='w', pady=(0,5))

        radio_container = customtkinter.CTkFrame(mode_frame, fg_color="transparent")
        radio_container.pack(side='top', fill='x', expand=False)

        self.radio_button_multiple = customtkinter.CTkRadioButton(
            radio_container,
            text='한글 선지',
            font=(self.default_font, 13),
            variable=self.요지_mode,
            value=1,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_multiple.grid(row=0, column=0, padx=(0, 10), pady=5, sticky="w")

        self.radio_button_single = customtkinter.CTkRadioButton(
            radio_container,
            text='영어 선지',
            font=(self.default_font, 13),
            variable=self.요지_mode,
            value=2,
            text_color="black",
            hover_color="#BB6C25",
            fg_color="#DDA15C"
        )
        self.radio_button_single.grid(row=0, column=1, padx=(10, 10), pady=5, sticky="w")





        # Action buttons at the bottom
        action_frame = customtkinter.CTkFrame(self.main_frame, fg_color="transparent")
        action_frame.pack(side='bottom', fill='x', expand=False, pady=(10,0))

        self.confirm_button = customtkinter.CTkButton(
            action_frame,
            text="확인",
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=self.on_confirm
        )
        self.confirm_button.pack(side='right', padx=5, pady=(0, 5))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()

    def on_confirm(self):

        # Save to JSON, including mode
        self.save_values_to_json(
            mode=self.요지_mode.get()
        )

        self.destroy()




    def load_values_from_json(self):
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    data = json.load(f)

                    saved_mode = data.get("요지mode", 1)
                    self.요지_mode.set(saved_mode)
            except:
                # If there's an error reading the file, fallback to defaults

                self.요지_mode.set(1)
        else:
            # No file, use defaults

            self.요지_mode.set(1)

    def save_values_to_json(self, mode=1):
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir, exist_ok=True)

        # Load existing data if any
        existing_data = {}
        if os.path.exists(self.make_questions_options_file_path):
            try:
                with open(self.make_questions_options_file_path, 'r', encoding='utf-8') as f:
                    existing_data = json.load(f)
            except:
                existing_data = {}

        # Update data with new values
        existing_data["요지mode"] = mode

        with open(self.make_questions_options_file_path, 'w', encoding='utf-8') as f:
            json.dump(existing_data, f, ensure_ascii=False, indent=4)







class QuestionOptions요약(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title('요약 옵션')
        self.geometry('450x100')
        self.minsize(450, 100)

        self.parent = parent
        self.main_frame_ref = main_frame  
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호
        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)


        if 실모flag == True:
            수동출제버튼텍스트 = "요약문 수동출제\n(현재 문항)"
            수동출제버튼커맨드 = self.on_manual_placement실모
        else:
            수동출제버튼텍스트 = "요약문 수동출제"
            수동출제버튼커맨드 = self.on_manual_placement

        self.manual_button = customtkinter.CTkButton(
            self.main_frame,
            text=수동출제버튼텍스트,
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=수동출제버튼커맨드
        )
        self.manual_button.pack(side='bottom', anchor='center', padx=5, pady=(0, 20))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()



    def on_manual_placement(self):
        
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.destroy()

        popup.grab_set()

    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체
        
        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.destroy()

        popup.grab_set()

        



class QuestionOptions영작1(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title('영작1 옵션')
        self.geometry('450x100')
        self.minsize(450, 100)

        self.parent = parent
        self.main_frame_ref = main_frame  
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호
        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)


        if 실모flag == True:
            수동출제버튼텍스트 = "영작1 수동출제\n(현재 문항)"
            수동출제버튼커맨드 = self.on_manual_placement실모
        else:
            수동출제버튼텍스트 = "영작1 수동출제"
            수동출제버튼커맨드 = self.on_manual_placement

        self.manual_button = customtkinter.CTkButton(
            self.main_frame,
            text=수동출제버튼텍스트,
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=수동출제버튼커맨드
        )
        self.manual_button.pack(side='bottom', anchor='center', padx=5, pady=(0, 20))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()



    def on_manual_placement(self):
        
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.destroy()

        popup.grab_set()

    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체
        
        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.destroy()

        popup.grab_set()

        




class QuestionOptions영작2(customtkinter.CTkToplevel):
    def __init__(self, parent, default_font="Arial", excel_file=None, main_frame=None, 실모flag=None, 수동출제할지문번호=None, row_idx=None, 유형=None, 난이도=None, passage_ids=None, manual_mocktest_popup=None):
        super().__init__(parent)
        self.title('영작2 옵션')
        self.geometry('450x100')
        self.minsize(450, 100)

        self.parent = parent
        self.main_frame_ref = main_frame  
        self.excel_file = excel_file
        self.실모flag = 실모flag
        self.수동출제할지문번호 = 수동출제할지문번호
        self.row_idx = row_idx
        self.유형 = 유형
        self.난이도 = 난이도
        self.passage_ids = passage_ids
        self.manual_mocktest_popup = manual_mocktest_popup


        self.default_font = default_font
        self.app_data_dir = Path.home() / ".my_application_data"
        self.make_questions_options_file_path = os.path.join(self.app_data_dir, "make_questions_options.json")

        # Main frame
        self.main_frame = customtkinter.CTkFrame(self, fg_color="transparent")
        self.main_frame.pack(pady=10, padx=20, fill='both', expand=True)


        if 실모flag == True:
            수동출제버튼텍스트 = "영작2 수동출제\n(현재 문항)"
            수동출제버튼커맨드 = self.on_manual_placement실모
        else:
            수동출제버튼텍스트 = "영작2 수동출제"
            수동출제버튼커맨드 = self.on_manual_placement

        self.manual_button = customtkinter.CTkButton(
            self.main_frame,
            text=수동출제버튼텍스트,
            font=(self.default_font, 13, 'bold'),
            width=160,
            height=40,
            fg_color="#FEF9E0",
            hover_color="#DDA15C",
            text_color="black",
            command=수동출제버튼커맨드
        )
        self.manual_button.pack(side='bottom', anchor='center', padx=5, pady=(0, 20))


        self.bind("<Escape>", self.on_popup_close)

    def on_popup_close(self, event=None):
        self.destroy()



    def on_manual_placement(self):
        
        popup = ManualQuestionPopup문제선택(
            parent=self.master,
            main_frame_ref=self.main_frame_ref,
            excel_file=self.excel_file,
            default_font=self.default_font,
            passage_ids=self.passage_ids,
            유형=self.유형,
            난이도=self.난이도,
        )
        self.destroy()

        popup.grab_set()

    def on_manual_placement실모(self):
        # Open the manual placement popup with 실모flag = True
        if self.main_frame_ref is None or self.실모flag is None or self.row_idx is None or self.수동출제할지문번호 is None:
            messagebox.showerror("오류", "필요한 정보가 부족합니다.")
            return

        def save_callback(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text):
            # Called by popup after user hits '확인'
            self.main_frame_ref.save_manual_placement_text실모(row_idx, 수동출제할지문번호, 유형, 난이도, edited_text)
            passageid유형난이도합본 = 수동출제할지문번호 + " // " + 유형 + "_" + 난이도
            self.manual_mocktest_popup.update_current_status_label수동실모(None, passageid유형난이도합본) #전체
        
        popup = ManualPlacementPopup(
            parent=self.master,
            default_font=self.default_font,
            excel_file=self.excel_file, 
            row_idx=self.row_idx,
            수동출제할지문번호=self.수동출제할지문번호,
            유형=self.유형,
            난이도=self.난이도,
            save_callback=save_callback,
            manual_edited_texts=self.main_frame_ref.manual_edited_texts,
            일반or실모="실모",
            manual_mocktest_popup=self.manual_mocktest_popup,
        )
        self.destroy()

        popup.grab_set()
